@@ -1,5 +1,5 @@
 use std::{
-    env, fs,
+    fs,
     path::{Path, PathBuf},
 };
 
@@ -9,7 +9,7 @@ use crate::{
 };
 
 pub(super) struct ArtifactsEmitter<'a> {
-    build: &'a Flutter,
+    build: &'a Flutter<'a>,
     flutter_out_dir: PathBuf,
     flutter_build_dir: PathBuf,
     artifacts_out_dir: PathBuf,
@@ -34,7 +34,7 @@ impl<'a> ArtifactsEmitter<'a> {
         let data_dir = self.artifacts_out_dir.join("data");
         if data_dir.exists() {
             std::fs::remove_dir_all(&data_dir)
-                .unwrap_or_else(|_| panic!("Failed to remove {:?}", data_dir));
+                .wrap_error(FileOperation::RemoveDir, || data_dir.clone())?;
         }
         let assets_dst_dir = mkdir(&data_dir, Some("flutter_assets"))?;
         let assets_src_dir = {
@@ -202,55 +202,21 @@ impl<'a> ArtifactsEmitter<'a> {
         }
     }
 
-    fn find_executable<P: AsRef<Path>>(exe_name: P) -> Option<PathBuf> {
-        env::var_os("PATH").and_then(|paths| {
-            env::split_paths(&paths)
-                .filter_map(|dir| {
-                    let full_path = dir.join(&exe_name);
-                    if full_path.is_file() {
-                        Some(full_path)
-                    } else {
-                        None
-                    }
-                })
-                .next()
-        })
-    }
-
-    pub(super) fn find_flutter_bin() -> Option<PathBuf> {
-        let executable = if cfg!(target_os = "windows") {
-            "flutter.bat"
-        } else {
-            "flutter"
-        };
-        let exe_path = Self::find_executable(executable).and_then(|p| p.canonicalize().ok());
-        exe_path.and_then(|p| p.parent().map(Path::to_owned))
-    }
-
-    fn find_flutter_bundled_artifacts_location() -> Option<PathBuf> {
-        Self::find_flutter_bin().map(|p| p.join("cache").join("artifacts").join("engine"))
-    }
-
-    fn find_local_engine_src_path() -> Option<PathBuf> {
-        Self::find_flutter_bin()
-            .and_then(|p| {
-                p.parent()
-                    .map(Path::to_owned)
-                    .and_then(|p| p.parent().map(Path::to_owned))
-            })
-            .map(|p| p.join("engine").join("src"))
+    fn find_flutter_bundled_artifacts_location(&self) -> BuildResult<PathBuf> {
+        Ok(self
+            .build
+            .options
+            .find_flutter_bin()?
+            .join("cache")
+            .join("artifacts")
+            .join("engine"))
     }
 
     pub(super) fn find_artifacts_location(&self, build_mode: &str) -> BuildResult<PathBuf> {
-        let path: Option<PathBuf> = match self.build.options.local_engine.as_ref() {
+        let path = match self.build.options.local_engine.as_ref() {
             Some(local_engine) => {
-                let engine_src_path = self
-                    .build
-                    .options
-                    .local_engine_src_path
-                    .clone()
-                    .or_else(Self::find_local_engine_src_path);
-                engine_src_path.map(|p| p.join("out").join(local_engine))
+                let engine_src_path = self.build.options.local_engine_src_path()?;
+                engine_src_path.join("out").join(local_engine)
             }
             None => {
                 let platform = match self.build.target_platform.as_str() {
@@ -268,13 +234,10 @@ impl<'a> ArtifactsEmitter<'a> {
                     (platform, "debug") => platform.into(),
                     (platform, mode) => format!("{}-{}", platform, mode),
                 };
-                Self::find_flutter_bundled_artifacts_location().map(|p| p.join(engine))
+                self.find_flutter_bundled_artifacts_location()?.join(engine)
             }
         };
 
-        let path = path.ok_or_else(|| BuildError::OtherError(
-            "Coud not determine flutter artifacts location; Please make sure that flutter is in PATH".into()))?;
-
         if !path.exists() {
             Err(BuildError::OtherError(format!(
                 "Flutter artifact location path at {:?} does not exist! Try running 'flutter precache'",
