@@ -13,30 +13,119 @@ use crate::{
     artifacts_emitter::ArtifactsEmitter,
     error::BuildError,
     plugins::Plugins,
-    util::{copy_to, get_artifacts_dir, run_command},
+    util::{copy_to, find_executable, get_artifacts_dir, run_command},
     BuildResult, FileOperation, IOResultExt,
 };
 
 // User configurable options during flutter build
 #[derive(Debug)]
-pub struct FlutterOptions {
+pub struct FlutterOptions<'a> {
     // lib/main.dart by default
-    pub target_file: PathBuf,
+    pub target_file: &'a Path,
 
-    pub local_engine: Option<String>,
-    pub local_engine_src_path: Option<PathBuf>,
+    // Custom Flutter location. If not specified, NativeShell build will try to find
+    // flutter executable in PATH and derive the location from there.
+    pub flutter_path: Option<&'a Path>,
+
+    // Name of local engine
+    pub local_engine: Option<&'a str>,
+
+    // Source path of local engine. If not specified, NativeShell will try to locate
+    // it relative to flutter path.
+    pub local_engine_src_path: Option<&'a Path>,
+
+    // macOS: Allow specifying extra pods to be built in addition to pods from
+    // Flutter plugins. For example: macos_extra_pods: &["pod 'Sparkle'"],
+    pub macos_extra_pods: &'a [&'a str],
 }
 
-impl Default for FlutterOptions {
+impl Default for FlutterOptions<'_> {
     fn default() -> Self {
         Self {
-            target_file: "lib/main.dart".into(),
+            target_file: "lib/main.dart".as_path(),
+            flutter_path: None,
             local_engine: None,
             local_engine_src_path: None,
+            macos_extra_pods: &[],
         }
     }
 }
 
+pub trait AsPath {
+    fn as_path(&self) -> &Path;
+}
+
+impl AsPath for str {
+    fn as_path(&self) -> &Path {
+        Path::new(self)
+    }
+}
+
+impl FlutterOptions<'_> {
+    pub(super) fn find_flutter_executable(&self) -> BuildResult<PathBuf> {
+        let executable = if cfg!(target_os = "windows") {
+            "flutter.bat"
+        } else {
+            "flutter"
+        };
+        match &self.flutter_path {
+            Some(path) => {
+                let out_dir: PathBuf = std::env::var("CARGO_MANIFEST_DIR").unwrap().into();
+                let path = out_dir.join(path);
+                let executable = path.join("bin").join(executable);
+                if executable.exists() {
+                    Ok(executable)
+                } else {
+                    Err(BuildError::FlutterPathInvalidError { path })
+                }
+            }
+            None => {
+                // Try FLUTER_ROOT if available
+                let flutter_root = std::env::var("FLUTTER_ROOT").ok();
+                if let Some(flutter_root) = flutter_root {
+                    let executable = Path::new(&flutter_root).join("bin").join(executable);
+                    if executable.exists() {
+                        return Ok(executable);
+                    }
+                }
+                let executable =
+                    find_executable(executable).ok_or(BuildError::FlutterNotFoundError)?;
+                let executable = executable
+                    .canonicalize()
+                    .wrap_error(FileOperation::Canonicalize, || executable)?;
+                let executable = simplified(&executable).into();
+                Ok(executable)
+            }
+        }
+    }
+
+    pub(super) fn find_flutter_bin(&self) -> BuildResult<PathBuf> {
+        let executable = self.find_flutter_executable()?;
+        Ok(executable.parent().unwrap().into())
+    }
+
+    pub(super) fn local_engine_src_path(&self) -> BuildResult<PathBuf> {
+        match &self.local_engine_src_path {
+            Some(path) => Ok(path.into()),
+            None => self.find_local_engine_src_path(),
+        }
+    }
+
+    fn find_local_engine_src_path(&self) -> BuildResult<PathBuf> {
+        let bin = self.find_flutter_bin()?;
+        let path = bin
+            .parent()
+            .and_then(|p| p.parent())
+            .map(|p| p.join("engine").join("src"));
+        if let Some(path) = path {
+            if path.exists() {
+                return Ok(path);
+            }
+        }
+        Err(BuildError::FlutterLocalEngineNotFound)
+    }
+}
+
 #[derive(Debug)]
 pub enum TargetOS {
     Mac,
@@ -45,17 +134,17 @@ pub enum TargetOS {
 }
 
 #[derive(Debug)]
-pub struct Flutter {
+pub struct Flutter<'a> {
     pub(super) root_dir: PathBuf,
     pub(super) out_dir: PathBuf,
-    pub(super) options: FlutterOptions,
+    pub(super) options: FlutterOptions<'a>,
     pub(super) build_mode: String,
     pub(super) target_os: TargetOS,
     pub(super) target_platform: String,
     pub(super) darwin_arch: Option<String>,
 }
 
-impl Flutter {
+impl Flutter<'_> {
     pub fn build(options: FlutterOptions) -> BuildResult<()> {
         let build = Flutter::new(options);
         build.do_build()
@@ -74,7 +163,7 @@ impl Flutter {
     }
 
     fn do_flutter_pub_get(&self) -> BuildResult<()> {
-        let mut command = self.create_flutter_command();
+        let mut command = self.create_flutter_command()?;
         command.arg("pub").arg("get");
         self.run_flutter_command(command)
     }
@@ -130,7 +219,7 @@ impl Flutter {
         to_dir: &Path,
         asset: &str,
     ) -> BuildResult<Option<PathBuf>> {
-        let mut segments = asset.split("/");
+        let mut segments = asset.split('/');
         if let Some(first) = segments.next() {
             if first != "packages" {
                 let asset = from_dir.join(first);
@@ -294,13 +383,14 @@ impl Flutter {
         Ok(())
     }
 
-    fn create_flutter_command(&self) -> Command {
+    fn create_flutter_command(&self) -> BuildResult<Command> {
+        let executable = self.options.find_flutter_executable()?;
         if cfg!(target_os = "windows") {
             let mut c = Command::new("cmd");
-            c.args(&["/C", "flutter"]);
-            c
+            c.arg("/C").arg(executable);
+            Ok(c)
         } else {
-            Command::new("flutter")
+            Ok(Command::new(executable))
         }
     }
 
@@ -332,16 +422,15 @@ impl Flutter {
             )],
         };
 
-        let mut command = self.create_flutter_command();
+        let mut command = self.create_flutter_command()?;
         command.current_dir(&working_dir);
 
         if let Some(local_engine) = &self.options.local_engine {
             command.arg(format!("--local-engine={}", local_engine));
-        }
-        if let Some(local_src_engine_path) = &self.options.local_engine_src_path {
+
             command.arg(format!(
                 "--local-engine-src-path={}",
-                local_src_engine_path.to_str().unwrap()
+                self.options.local_engine_src_path()?.to_slash_lossy()
             ));
         }
         command
@@ -370,9 +459,9 @@ impl Flutter {
     ) -> BuildResult<()> {
         let artifacts_dir = get_artifacts_dir()?;
         let flutter_out_root = self.out_dir.join("flutter");
-        let emitter = ArtifactsEmitter::new(&self, flutter_out_root, artifacts_dir)?;
+        let emitter = ArtifactsEmitter::new(self, flutter_out_root, artifacts_dir)?;
 
-        let plugins = Plugins::new(&self, &emitter);
+        let plugins = Plugins::new(self, &emitter);
         plugins.process()?;
 
         match self.target_os {
@@ -453,8 +542,9 @@ impl Flutter {
         if self.options.local_engine.is_some() {
             return Ok(());
         }
-        let engine_version = ArtifactsEmitter::find_flutter_bin()
-            .ok_or_else(|| BuildError::OtherError("Couldn't find Flutter installation".into()))?
+        let engine_version = self
+            .options
+            .find_flutter_bin()?
             .join("internal")
             .join("engine.version");
 
@@ -471,7 +561,7 @@ impl Flutter {
         }
 
         // need to run flutter precache
-        let mut command = self.create_flutter_command();
+        let mut command = self.create_flutter_command()?;
         command
             .arg("precache")
             .arg("-v")
